@@ -1,10 +1,21 @@
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+
+
 use itertools::Itertools;
 use pulldown_cmark;
-use rustc::lint::*;
-use syntax::ast;
-use syntax::codemap::{BytePos, Span};
-use syntax_pos::Pos;
-use utils::span_lint;
+use crate::rustc::lint::{EarlyContext, EarlyLintPass, LintArray, LintPass};
+use crate::rustc::{declare_tool_lint, lint_array};
+use crate::syntax::ast;
+use crate::syntax::source_map::{BytePos, Span};
+use crate::syntax_pos::Pos;
+use crate::utils::span_lint;
 use url::Url;
 
 /// **What it does:** Checks for the presence of `_`, `::` or camel-case words
@@ -51,11 +62,11 @@ impl LintPass for Doc {
 }
 
 impl EarlyLintPass for Doc {
-    fn check_crate(&mut self, cx: &EarlyContext, krate: &ast::Crate) {
+    fn check_crate(&mut self, cx: &EarlyContext<'_>, krate: &ast::Crate) {
         check_attrs(cx, &self.valid_idents, &krate.attrs);
     }
 
-    fn check_item(&mut self, cx: &EarlyContext, item: &ast::Item) {
+    fn check_item(&mut self, cx: &EarlyContext<'_>, item: &ast::Item) {
         check_attrs(cx, &self.valid_idents, &item.attrs);
     }
 }
@@ -85,7 +96,7 @@ impl<'a> Iterator for Parser<'a> {
 /// `syntax::parse::lexer::comments::strip_doc_comment_decoration` because we
 /// need to keep track of
 /// the spans but this function is inspired from the later.
-#[allow(cast_possible_truncation)]
+#[allow(clippy::cast_possible_truncation)]
 pub fn strip_doc_comment_decoration(comment: &str, span: Span) -> (String, Vec<(usize, Span)>) {
     // one-line comments lose their prefix
     const ONELINERS: &[&str] = &["///!", "///", "//!", "//"];
@@ -138,7 +149,7 @@ pub fn strip_doc_comment_decoration(comment: &str, span: Span) -> (String, Vec<(
     panic!("not a doc-comment: {}", comment);
 }
 
-pub fn check_attrs<'a>(cx: &EarlyContext, valid_idents: &[String], attrs: &'a [ast::Attribute]) {
+pub fn check_attrs<'a>(cx: &EarlyContext<'_>, valid_idents: &[String], attrs: &'a [ast::Attribute]) {
     let mut doc = String::new();
     let mut spans = vec![];
 
@@ -150,11 +161,9 @@ pub fn check_attrs<'a>(cx: &EarlyContext, valid_idents: &[String], attrs: &'a [a
                 spans.extend_from_slice(&current_spans);
                 doc.push_str(&current);
             }
-        } else if let Some(name) = attr.name() {
+        } else if attr.name() == "doc" {
             // ignore mix of sugared and non-sugared doc
-            if name == "doc" {
-                return;
-            }
+            return;
         }
     }
 
@@ -187,7 +196,7 @@ pub fn check_attrs<'a>(cx: &EarlyContext, valid_idents: &[String], attrs: &'a [a
 }
 
 fn check_doc<'a, Events: Iterator<Item = (usize, pulldown_cmark::Event<'a>)>>(
-    cx: &EarlyContext,
+    cx: &EarlyContext<'_>,
     valid_idents: &[String],
     docs: Events,
     spans: &[(usize, Span)],
@@ -233,7 +242,7 @@ fn check_doc<'a, Events: Iterator<Item = (usize, pulldown_cmark::Event<'a>)>>(
     }
 }
 
-fn check_text(cx: &EarlyContext, valid_idents: &[String], text: &str, span: Span) {
+fn check_text(cx: &EarlyContext<'_>, valid_idents: &[String], text: &str, span: Span) {
     for word in text.split_whitespace() {
         // Trim punctuation as in `some comment (see foo::bar).`
         //                                                   ^^
@@ -256,7 +265,7 @@ fn check_text(cx: &EarlyContext, valid_idents: &[String], text: &str, span: Span
     }
 }
 
-fn check_word(cx: &EarlyContext, word: &str, span: Span) {
+fn check_word(cx: &EarlyContext<'_>, word: &str, span: Span) {
     /// Checks if a string is camel-case, ie. contains at least two uppercase
     /// letter (`Clippy` is
     /// ok) and one lower-case letter (`NASA` is ok). Plural are also excluded
