@@ -1,7 +1,18 @@
-use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use rustc::hir::def::Def;
-use rustc::hir::{BiAnd, BiOr, BlockCheckMode, Expr, Expr_, Stmt, StmtSemi, UnsafeSource};
-use utils::{has_drop, in_macro, snippet_opt, span_lint, span_lint_and_sugg};
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+
+
+use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use crate::rustc::{declare_tool_lint, lint_array};
+use crate::rustc::hir::def::Def;
+use crate::rustc::hir::{BinOpKind, BlockCheckMode, Expr, ExprKind, Stmt, StmtKind, UnsafeSource};
+use crate::utils::{has_drop, in_macro, snippet_opt, span_lint, span_lint_and_sugg};
 use std::ops::Deref;
 
 /// **What it does:** Checks for statements which have no effect.
@@ -40,31 +51,31 @@ declare_clippy_lint! {
     "outer expressions with no effect"
 }
 
-fn has_no_effect(cx: &LateContext, expr: &Expr) -> bool {
+fn has_no_effect(cx: &LateContext<'_, '_>, expr: &Expr) -> bool {
     if in_macro(expr.span) {
         return false;
     }
     match expr.node {
-        Expr_::ExprLit(..) | Expr_::ExprClosure(.., _) => true,
-        Expr_::ExprPath(..) => !has_drop(cx, expr),
-        Expr_::ExprIndex(ref a, ref b) | Expr_::ExprBinary(_, ref a, ref b) => {
+        ExprKind::Lit(..) | ExprKind::Closure(.., _) => true,
+        ExprKind::Path(..) => !has_drop(cx, expr),
+        ExprKind::Index(ref a, ref b) | ExprKind::Binary(_, ref a, ref b) => {
             has_no_effect(cx, a) && has_no_effect(cx, b)
         },
-        Expr_::ExprArray(ref v) | Expr_::ExprTup(ref v) => v.iter().all(|val| has_no_effect(cx, val)),
-        Expr_::ExprRepeat(ref inner, _) |
-        Expr_::ExprCast(ref inner, _) |
-        Expr_::ExprType(ref inner, _) |
-        Expr_::ExprUnary(_, ref inner) |
-        Expr_::ExprField(ref inner, _) |
-        Expr_::ExprAddrOf(_, ref inner) |
-        Expr_::ExprBox(ref inner) => has_no_effect(cx, inner),
-        Expr_::ExprStruct(_, ref fields, ref base) => {
+        ExprKind::Array(ref v) | ExprKind::Tup(ref v) => v.iter().all(|val| has_no_effect(cx, val)),
+        ExprKind::Repeat(ref inner, _) |
+        ExprKind::Cast(ref inner, _) |
+        ExprKind::Type(ref inner, _) |
+        ExprKind::Unary(_, ref inner) |
+        ExprKind::Field(ref inner, _) |
+        ExprKind::AddrOf(_, ref inner) |
+        ExprKind::Box(ref inner) => has_no_effect(cx, inner),
+        ExprKind::Struct(_, ref fields, ref base) => {
             !has_drop(cx, expr) && fields.iter().all(|field| has_no_effect(cx, &field.expr)) && match *base {
                 Some(ref base) => has_no_effect(cx, base),
                 None => true,
             }
         },
-        Expr_::ExprCall(ref callee, ref args) => if let Expr_::ExprPath(ref qpath) = callee.node {
+        ExprKind::Call(ref callee, ref args) => if let ExprKind::Path(ref qpath) = callee.node {
             let def = cx.tables.qpath_def(qpath, callee.hir_id);
             match def {
                 Def::Struct(..) | Def::Variant(..) | Def::StructCtor(..) | Def::VariantCtor(..) => {
@@ -75,7 +86,7 @@ fn has_no_effect(cx: &LateContext, expr: &Expr) -> bool {
         } else {
             false
         },
-        Expr_::ExprBlock(ref block, _) => {
+        ExprKind::Block(ref block, _) => {
             block.stmts.is_empty() && if let Some(ref expr) = block.expr {
                 has_no_effect(cx, expr)
             } else {
@@ -97,7 +108,7 @@ impl LintPass for Pass {
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Pass {
     fn check_stmt(&mut self, cx: &LateContext<'a, 'tcx>, stmt: &'tcx Stmt) {
-        if let StmtSemi(ref expr, _) = stmt.node {
+        if let StmtKind::Semi(ref expr, _) = stmt.node {
             if has_no_effect(cx, expr) {
                 span_lint(cx, NO_EFFECT, stmt.span, "statement with no effect");
             } else if let Some(reduced) = reduce_expression(cx, expr) {
@@ -127,24 +138,24 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Pass {
 }
 
 
-fn reduce_expression<'a>(cx: &LateContext, expr: &'a Expr) -> Option<Vec<&'a Expr>> {
+fn reduce_expression<'a>(cx: &LateContext<'_, '_>, expr: &'a Expr) -> Option<Vec<&'a Expr>> {
     if in_macro(expr.span) {
         return None;
     }
     match expr.node {
-        Expr_::ExprIndex(ref a, ref b) => Some(vec![&**a, &**b]),
-        Expr_::ExprBinary(ref binop, ref a, ref b) if binop.node != BiAnd && binop.node != BiOr => {
+        ExprKind::Index(ref a, ref b) => Some(vec![&**a, &**b]),
+        ExprKind::Binary(ref binop, ref a, ref b) if binop.node != BinOpKind::And && binop.node != BinOpKind::Or => {
             Some(vec![&**a, &**b])
         },
-        Expr_::ExprArray(ref v) | Expr_::ExprTup(ref v) => Some(v.iter().collect()),
-        Expr_::ExprRepeat(ref inner, _) |
-        Expr_::ExprCast(ref inner, _) |
-        Expr_::ExprType(ref inner, _) |
-        Expr_::ExprUnary(_, ref inner) |
-        Expr_::ExprField(ref inner, _) |
-        Expr_::ExprAddrOf(_, ref inner) |
-        Expr_::ExprBox(ref inner) => reduce_expression(cx, inner).or_else(|| Some(vec![inner])),
-        Expr_::ExprStruct(_, ref fields, ref base) => if has_drop(cx, expr) {
+        ExprKind::Array(ref v) | ExprKind::Tup(ref v) => Some(v.iter().collect()),
+        ExprKind::Repeat(ref inner, _) |
+        ExprKind::Cast(ref inner, _) |
+        ExprKind::Type(ref inner, _) |
+        ExprKind::Unary(_, ref inner) |
+        ExprKind::Field(ref inner, _) |
+        ExprKind::AddrOf(_, ref inner) |
+        ExprKind::Box(ref inner) => reduce_expression(cx, inner).or_else(|| Some(vec![inner])),
+        ExprKind::Struct(_, ref fields, ref base) => if has_drop(cx, expr) {
             None
         } else {
             Some(
@@ -156,7 +167,7 @@ fn reduce_expression<'a>(cx: &LateContext, expr: &'a Expr) -> Option<Vec<&'a Exp
                     .collect(),
             )
         },
-        Expr_::ExprCall(ref callee, ref args) => if let Expr_::ExprPath(ref qpath) = callee.node {
+        ExprKind::Call(ref callee, ref args) => if let ExprKind::Path(ref qpath) = callee.node {
             let def = cx.tables.qpath_def(qpath, callee.hir_id);
             match def {
                 Def::Struct(..) | Def::Variant(..) | Def::StructCtor(..) | Def::VariantCtor(..)
@@ -169,7 +180,7 @@ fn reduce_expression<'a>(cx: &LateContext, expr: &'a Expr) -> Option<Vec<&'a Exp
         } else {
             None
         },
-        Expr_::ExprBlock(ref block, _) => {
+        ExprKind::Block(ref block, _) => {
             if block.stmts.is_empty() {
                 block.expr.as_ref().and_then(|e| {
                     match block.rules {
