@@ -1,11 +1,21 @@
-use rustc::lint::*;
-use rustc::{declare_lint, lint_array};
-use rustc::hir;
-use rustc::hir::intravisit::{walk_expr, walk_fn, FnKind, NestedVisitorMap, Visitor};
-use std::collections::HashMap;
-use syntax::ast;
-use syntax::codemap::Span;
-use syntax::symbol::LocalInternedString;
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+
+
+use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use crate::rustc::{declare_tool_lint, lint_array};
+use crate::rustc::hir;
+use crate::rustc::hir::intravisit::{walk_expr, walk_fn, FnKind, NestedVisitorMap, Visitor};
+use crate::rustc_data_structures::fx::FxHashMap;
+use crate::syntax::ast;
+use crate::syntax::source_map::Span;
+use crate::syntax::symbol::LocalInternedString;
 use crate::utils::{in_macro, span_lint};
 
 /// **What it does:** Checks for unused labels.
@@ -31,7 +41,7 @@ declare_clippy_lint! {
 pub struct UnusedLabel;
 
 struct UnusedLabelVisitor<'a, 'tcx: 'a> {
-    labels: HashMap<LocalInternedString, Span>,
+    labels: FxHashMap<LocalInternedString, Span>,
     cx: &'a LateContext<'a, 'tcx>,
 }
 
@@ -57,7 +67,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for UnusedLabel {
 
         let mut v = UnusedLabelVisitor {
             cx,
-            labels: HashMap::new(),
+            labels: FxHashMap::default(),
         };
         walk_fn(&mut v, kind, decl, body.id(), span, fn_id);
 
