@@ -1,19 +1,29 @@
-use rustc::lint::*;
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
 
-use rustc::hir::def::Def;
-use rustc::hir::*;
-use rustc::middle::expr_use_visitor::*;
-use rustc::middle::mem_categorization::cmt_;
-use rustc::middle::mem_categorization::Categorization;
-use rustc::ty;
-use std::collections::HashSet;
-use syntax::ast::NodeId;
-use syntax::codemap::Span;
+
+use crate::rustc::lint::LateContext;
+
+use crate::rustc::hir::def::Def;
+use crate::rustc::hir::*;
+use crate::rustc::middle::expr_use_visitor::*;
+use crate::rustc::middle::mem_categorization::cmt_;
+use crate::rustc::middle::mem_categorization::Categorization;
+use crate::rustc::ty;
+use crate::rustc_data_structures::fx::FxHashSet;
+use crate::syntax::ast::NodeId;
+use crate::syntax::source_map::Span;
 
 /// Returns a set of mutated local variable ids or None if mutations could not be determined.
-pub fn mutated_variables<'a, 'tcx: 'a>(expr: &'tcx Expr, cx: &'a LateContext<'a, 'tcx>) -> Option<HashSet<NodeId>> {
+pub fn mutated_variables<'a, 'tcx: 'a>(expr: &'tcx Expr, cx: &'a LateContext<'a, 'tcx>) -> Option<FxHashSet<NodeId>> {
     let mut delegate = MutVarsDelegate {
-        used_mutably: HashSet::new(),
+        used_mutably: FxHashSet::default(),
         skip: false,
     };
     let def_id = def_id::DefId::local(expr.hir_id.owner);
@@ -39,11 +49,12 @@ pub fn is_potentially_mutated<'a, 'tcx: 'a>(
 }
 
 struct MutVarsDelegate {
-    used_mutably: HashSet<NodeId>,
+    used_mutably: FxHashSet<NodeId>,
     skip: bool,
 }
 
 impl<'tcx> MutVarsDelegate {
+    #[allow(clippy::similar_names)]
     fn update(&mut self, cat: &'tcx Categorization<'_>) {
         match *cat {
             Categorization::Local(id) => {
