@@ -1,11 +1,23 @@
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+
+
+
+
 use std::borrow::Cow;
 
-
+#[allow(clippy::trivially_copy_pass_by_ref)]
 fn x(y: &i32) -> i32 {
     *y
 }
 
-#[warn(clippy)]
+#[warn(clippy::all, clippy::needless_borrow)]
 #[allow(unused_variables)]
 fn main() {
     let a = 5;
@@ -42,7 +54,7 @@ trait Trait {}
 impl<'a> Trait for &'a str {}
 
 fn h(_: &Trait) {}
-
+#[warn(clippy::needless_borrow)]
 #[allow(dead_code)]
 fn issue_1432() {
     let mut v = Vec::<String>::new();
@@ -51,12 +63,3 @@ fn issue_1432() {
 
     let _ = v.iter().filter(|&a| a.is_empty());
 }
-
-#[allow(dead_code)]
-fn test_cow_with_ref(c: &Cow<[i32]>) {
-}
-
-#[allow(dead_code)]
-fn test_cow(c: Cow<[i32]>) {
-    let _c = c;
-}
