@@ -1,31 +1,55 @@
-use rustc::{ty, mir};
+use std::collections::BTreeMap;
 
-use super::{TlsKey, TlsEntry, EvalResult, EvalErrorKind, Scalar, ScalarExt, Memory, Evaluator,
-            Place, StackPopCleanup, EvalContext};
+use rustc_target::abi::LayoutOf;
+use rustc::{ty, ty::layout::HasDataLayout, mir};
 
-pub trait MemoryExt<'tcx> {
-    fn create_tls_key(&mut self, dtor: Option<ty::Instance<'tcx>>) -> TlsKey;
-    fn delete_tls_key(&mut self, key: TlsKey) -> EvalResult<'tcx>;
-    fn load_tls(&mut self, key: TlsKey) -> EvalResult<'tcx, Scalar>;
-    fn store_tls(&mut self, key: TlsKey, new_data: Scalar) -> EvalResult<'tcx>;
-    fn fetch_tls_dtor(
-        &mut self,
-        key: Option<TlsKey>,
-    ) -> EvalResult<'tcx, Option<(ty::Instance<'tcx>, Scalar, TlsKey)>>;
+use super::{
+    EvalResult, EvalErrorKind, StackPopCleanup,
+    MPlaceTy, Scalar, Borrow,
+};
+
+pub type TlsKey = u128;
+
+#[derive(Copy, Clone, Debug)]
+pub struct TlsEntry<'tcx> {
+    pub(crate) data: Scalar<Borrow>, // Will eventually become a map from thread IDs to `Scalar`s, if we ever support more than one thread.
+    pub(crate) dtor: Option<ty::Instance<'tcx>>,
+}
+
+#[derive(Debug)]
+pub struct TlsData<'tcx> {
+    /// The Key to use for the next thread-local allocation.
+    pub(crate) next_key: TlsKey,
+
+    /// pthreads-style thread-local storage.
+    pub(crate) keys: BTreeMap<TlsKey, TlsEntry<'tcx>>,
+}
+
+impl<'tcx> Default for TlsData<'tcx> {
+    fn default() -> Self {
+        TlsData {
+            next_key: 1, // start with 1 as we must not use 0 on Windows
+            keys: Default::default(),
+        }
+    }
 }
 
 pub trait EvalContextExt<'tcx> {
     fn run_tls_dtors(&mut self) -> EvalResult<'tcx>;
 }
 
-impl<'a, 'mir, 'tcx: 'mir + 'a> MemoryExt<'tcx> for Memory<'a, 'mir, 'tcx, Evaluator<'tcx>> {
-    fn create_tls_key(&mut self, dtor: Option<ty::Instance<'tcx>>) -> TlsKey {
-        let new_key = self.data.next_thread_local;
-        self.data.next_thread_local += 1;
-        self.data.thread_local.insert(
+impl<'tcx> TlsData<'tcx> {
+    pub fn create_tls_key(
+        &mut self,
+        dtor: Option<ty::Instance<'tcx>>,
+        cx: impl HasDataLayout,
+    ) -> TlsKey {
+        let new_key = self.next_key;
+        self.next_key += 1;
+        self.keys.insert(
             new_key,
             TlsEntry {
-                data: Scalar::null(),
+                data: Scalar::ptr_null(cx).into(),
                 dtor,
             },
         );
@@ -33,8 +57,8 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> MemoryExt<'tcx> for Memory<'a, 'mir, 'tcx, Evalu
         new_key
     }
 
-    fn delete_tls_key(&mut self, key: TlsKey) -> EvalResult<'tcx> {
-        match self.data.thread_local.remove(&key) {
+    pub fn delete_tls_key(&mut self, key: TlsKey) -> EvalResult<'tcx> {
+        match self.keys.remove(&key) {
             Some(_) => {
                 trace!("TLS key {} removed", key);
                 Ok(())
@@ -43,8 +67,8 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> MemoryExt<'tcx> for Memory<'a, 'mir, 'tcx, Evalu
         }
     }
 
-    fn load_tls(&mut self, key: TlsKey) -> EvalResult<'tcx, Scalar> {
-        match self.data.thread_local.get(&key) {
+    pub fn load_tls(&mut self, key: TlsKey) -> EvalResult<'tcx, Scalar<Borrow>> {
+        match self.keys.get(&key) {
             Some(&TlsEntry { data, .. }) => {
                 trace!("TLS key {} loaded: {:?}", key, data);
                 Ok(data)
@@ -53,8 +77,8 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> MemoryExt<'tcx> for Memory<'a, 'mir, 'tcx, Evalu
         }
     }
 
-    fn store_tls(&mut self, key: TlsKey, new_data: Scalar) -> EvalResult<'tcx> {
-        match self.data.thread_local.get_mut(&key) {
+    pub fn store_tls(&mut self, key: TlsKey, new_data: Scalar<Borrow>) -> EvalResult<'tcx> {
+        match self.keys.get_mut(&key) {
             Some(&mut TlsEntry { ref mut data, .. }) => {
                 trace!("TLS key {} stored: {:?}", key, new_data);
                 *data = new_data;
@@ -85,10 +109,11 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> MemoryExt<'tcx> for Memory<'a, 'mir, 'tcx, Evalu
     fn fetch_tls_dtor(
         &mut self,
         key: Option<TlsKey>,
-    ) -> EvalResult<'tcx, Option<(ty::Instance<'tcx>, Scalar, TlsKey)>> {
+        cx: impl HasDataLayout,
+    ) -> Option<(ty::Instance<'tcx>, Scalar<Borrow>, TlsKey)> {
         use std::collections::Bound::*;
 
-        let thread_local = &mut self.data.thread_local;
+        let thread_local = &mut self.keys;
         let start = match key {
             Some(key) => Excluded(key),
             None => Unbounded,
@@ -96,47 +121,47 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> MemoryExt<'tcx> for Memory<'a, 'mir, 'tcx, Evalu
         for (&key, &mut TlsEntry { ref mut data, dtor }) in
             thread_local.range_mut((start, Unbounded))
         {
-            if !data.is_null()? {
+            if !data.is_null() {
                 if let Some(dtor) = dtor {
                     let ret = Some((dtor, *data, key));
-                    *data = Scalar::null();
-                    return Ok(ret);
+                    *data = Scalar::ptr_null(cx);
+                    return ret;
                 }
             }
         }
-        Ok(None)
+        None
     }
 }
 
-impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx> for EvalContext<'a, 'mir, 'tcx, Evaluator<'tcx>> {
+impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx> for super::MiriEvalContext<'a, 'mir, 'tcx> {
     fn run_tls_dtors(&mut self) -> EvalResult<'tcx> {
-        let mut dtor = self.memory.fetch_tls_dtor(None)?;
+        let mut dtor = self.machine.tls.fetch_tls_dtor(None, *self.tcx);
         // FIXME: replace loop by some structure that works with stepping
         while let Some((instance, ptr, key)) = dtor {
             trace!("Running TLS dtor {:?} on {:?}", instance, ptr);
             // TODO: Potentially, this has to support all the other possible instances?
             // See eval_fn_call in interpret/terminator/mod.rs
             let mir = self.load_mir(instance.def)?;
+            let ret_place = MPlaceTy::dangling(self.layout_of(self.tcx.mk_unit())?, &self).into();
             self.push_stack_frame(
                 instance,
                 mir.span,
                 mir,
-                Place::undef(),
-                StackPopCleanup::None,
+                Some(ret_place),
+                StackPopCleanup::None { cleanup: true },
             )?;
             let arg_local = self.frame().mir.args_iter().next().ok_or_else(
                 || EvalErrorKind::AbiViolation("TLS dtor does not take enough arguments.".to_owned()),
             )?;
             let dest = self.eval_place(&mir::Place::Local(arg_local))?;
-            let ty = self.tcx.mk_mut_ptr(self.tcx.types.u8);
-            self.write_ptr(dest, ptr, ty)?;
+            self.write_scalar(ptr, dest)?;
 
             // step until out of stackframes
-            while self.step()? {}
+            self.run()?;
 
-            dtor = match self.memory.fetch_tls_dtor(Some(key))? {
+            dtor = match self.machine.tls.fetch_tls_dtor(Some(key), *self.tcx) {
                 dtor @ Some(_) => dtor,
-                None => self.memory.fetch_tls_dtor(None)?,
+                None => self.machine.tls.fetch_tls_dtor(None, *self.tcx),
             };
         }
         // FIXME: On a windows target, call `unsafe extern "system" fn on_tls_callback`.
