@@ -10,47 +10,43 @@
 
 //! Requests that the RLS can respond to.
 
-use actions::InitActionContext;
-use data;
-use url::Url;
-use vfs::FileContents;
+use crate::actions::InitActionContext;
+use itertools::Itertools;
+use log::{debug, trace, warn};
 use racer;
-use rustfmt::{FileName, format_input, Input as FmtInput};
-use rustfmt::file_lines::{FileLines, Range as RustfmtRange};
+use rls_data as data;
+use rls_span as span;
+use rls_vfs::FileContents;
+use rustfmt_nightly::{Edition as RustfmtEdition, FileLines, FileName, Range as RustfmtRange};
+use serde_derive::{Deserialize, Serialize};
 use serde_json;
-use span;
+use url::Url;
 
-use actions::work_pool;
-use actions::work_pool::WorkDescription;
-use lsp_data;
-use lsp_data::*;
-use server;
-use server::{Ack, Output, Request, RequestAction, ResponseError};
+use crate::actions::hover;
+use crate::actions::run::collect_run_actions;
+use crate::actions::work_pool;
+use crate::actions::work_pool::WorkDescription;
+use crate::build::Edition;
+use crate::lsp_data;
+use crate::lsp_data::*;
+use crate::server;
+use crate::server::{Ack, Output, Request, RequestAction, ResponseError, ResponseWithMessage};
 use jsonrpc_core::types::ErrorCode;
+use rls_analysis::SymbolQuery;
 
-use lsp_data::request::ApplyWorkspaceEdit;
-pub use lsp_data::request::{
-    WorkspaceSymbol,
-    DocumentSymbol as Symbols,
-    HoverRequest as Hover,
-    GotoDefinition as Definition,
-    References,
-    Completion,
-    DocumentHighlightRequest as DocumentHighlight,
-    Rename,
-    ExecuteCommand,
-    CodeActionRequest as CodeAction,
-    Formatting,
-    RangeFormatting,
-    ResolveCompletionItem as ResolveCompletion,
+use crate::lsp_data::request::ApplyWorkspaceEdit;
+pub use crate::lsp_data::request::{
+    CodeActionRequest as CodeAction, CodeLensRequest, Completion,
+    DocumentHighlightRequest as DocumentHighlight, DocumentSymbol as Symbols, ExecuteCommand,
+    Formatting, GotoDefinition as Definition, GotoImplementation as Implementation,
+    HoverRequest as Hover, RangeFormatting, References, Rename,
+    ResolveCompletionItem as ResolveCompletion, WorkspaceSymbol,
 };
-pub use lsp_data::FindImpls;
 
 use std::collections::HashMap;
 use std::path::Path;
 use std::sync::atomic::Ordering;
 
-
 /// Represent the result of a deglob action for a single wildcard import.
 ///
 /// The `location` is the position of the wildcard.
@@ -75,11 +71,13 @@ impl RequestAction for WorkspaceSymbol {
         params: Self::Params,
     ) -> Result<Self::Response, ResponseError> {
         let analysis = ctx.analysis;
+        let query = SymbolQuery::subsequence(&params.query).limit(512);
+        let defs = analysis.query_defs(query).unwrap_or_else(|_| vec![]);
 
-        let defs = analysis.matching_defs(&params.query).unwrap_or_else(|_| vec![]);
-
-        Ok(
-            defs.into_iter()
+        Ok(defs.into_iter()
+                // Sometimes analysis will return duplicate symbols
+                // for the same location, fix that up.
+                .unique_by(|d| (d.span.clone(), d.name.clone()))
                 .map(|d| {
                     SymbolInformation {
                         name: d.name,
@@ -90,8 +88,7 @@ impl RequestAction for WorkspaceSymbol {
                             .map(|parent| parent.name),
                     }
                 })
-                .collect(),
-        )
+                .collect())
     }
 }
 
@@ -112,21 +109,20 @@ impl RequestAction for Symbols {
 
         let symbols = analysis.symbols(&file_path).unwrap_or_else(|_| vec![]);
 
-        Ok(
-            symbols
-                .into_iter()
-                .map(|s| {
-                    SymbolInformation {
-                        name: s.name,
-                        kind: source_kind_from_def_kind(s.kind),
-                        location: ls_util::rls_to_location(&s.span),
-                        container_name: s.parent
-                            .and_then(|id| analysis.get_def(id).ok())
-                            .map(|parent| parent.name),
-                    }
-                })
-                .collect(),
-        )
+        Ok(symbols
+            .into_iter()
+            .filter(|s| {
+                let range = ls_util::rls_to_range(s.span.range);
+                range.start != range.end
+            }).map(|s| SymbolInformation {
+                name: s.name,
+                kind: source_kind_from_def_kind(s.kind),
+                location: ls_util::rls_to_location(&s.span),
+                container_name: s
+                    .parent
+                    .and_then(|id| analysis.get_def(id).ok())
+                    .map(|parent| parent.name),
+            }).collect())
     }
 }
 
@@ -144,34 +140,16 @@ impl RequestAction for Hover {
         ctx: InitActionContext,
         params: Self::Params,
     ) -> Result<Self::Response, ResponseError> {
-        let file_path = parse_file_path!(&params.text_document.uri, "hover")?;
-        let span = ctx.convert_pos_to_span(file_path, params.position);
+        let tooltip = hover::tooltip(&ctx, &params)?;
 
-        trace!("hover: {:?}", span);
-
-        let analysis = ctx.analysis;
-        let ty = analysis.show_type(&span).unwrap_or_else(|_| String::new());
-        let docs = analysis.docs(&span).unwrap_or_else(|_| String::new());
-        let doc_url = analysis.doc_url(&span).unwrap_or_else(|_| String::new());
-
-        let mut contents = vec![];
-        if !docs.is_empty() {
-            contents.push(MarkedString::from_markdown(docs));
-        }
-        if !doc_url.is_empty() {
-            contents.push(MarkedString::from_markdown(doc_url));
-        }
-        if !ty.is_empty() {
-            contents.push(MarkedString::from_language_code("rust".into(), ty));
-        }
         Ok(lsp_data::Hover {
-            contents: HoverContents::Array(contents),
+            contents: HoverContents::Array(tooltip),
             range: None, // TODO: maybe add?
         })
     }
 }
 
-impl RequestAction for FindImpls {
+impl RequestAction for Implementation {
     type Response = Vec<Location>;
 
     fn fallback_response() -> Result<Self::Response, ResponseError> {
@@ -219,20 +197,22 @@ impl RequestAction for Definition {
         // Save-analysis thread.
         let file_path = parse_file_path!(&params.text_document.uri, "goto_def")?;
         let span = ctx.convert_pos_to_span(file_path.clone(), params.position);
-        let analysis = ctx.analysis;
-        let vfs = ctx.vfs;
+        let analysis = ctx.analysis.clone();
 
         // If configured start racer concurrently and fallback to racer result
         let racer_receiver = {
             if ctx.config.lock().unwrap().goto_def_racer_fallback {
-                Some(work_pool::receive_from_thread(move || {
-                    let cache = racer::FileCache::new(vfs);
-                    let session = racer::Session::new(&cache);
-                    let location = pos_to_racer_location(params.position);
+                Some(work_pool::receive_from_thread(
+                    move || {
+                        let cache = ctx.racer_cache();
+                        let session = ctx.racer_session(&cache);
+                        let location = pos_to_racer_location(params.position);
 
-                    racer::find_definition(file_path, location, &session)
-                        .and_then(|rm| location_from_racer_match(&rm))
-                }, WorkDescription("textDocument/definition-racer")))
+                        racer::find_definition(file_path, location, &session)
+                            .and_then(|rm| location_from_racer_match(&rm))
+                    },
+                    WorkDescription("textDocument/definition-racer"),
+                ))
             } else {
                 None
             }
@@ -276,19 +256,19 @@ impl RequestAction for References {
         let file_path = parse_file_path!(&params.text_document.uri, "find_all_refs")?;
         let span = ctx.convert_pos_to_span(file_path, params.position);
 
-        let result = match ctx.analysis
-            .find_all_refs(&span, params.context.include_declaration, false)
-        {
-            Ok(t) => t,
-            _ => vec![],
-        };
+        let result =
+            match ctx
+                .analysis
+                .find_all_refs(&span, params.context.include_declaration, false)
+            {
+                Ok(t) => t,
+                _ => vec![],
+            };
 
-        Ok(
-            result
-                .iter()
-                .map(|item| ls_util::rls_to_location(item))
-                .collect(),
-        )
+        Ok(result
+            .iter()
+            .map(|item| ls_util::rls_to_location(item))
+            .collect())
     }
 }
 
@@ -307,33 +287,32 @@ impl RequestAction for Completion {
             return Self::fallback_response();
         }
 
-        let vfs = ctx.vfs;
         let file_path = parse_file_path!(&params.text_document.uri, "complete")?;
 
-        let cache = racer::FileCache::new(vfs);
-        let session = racer::Session::new(&cache);
+        let cache = ctx.racer_cache();
+        let session = ctx.racer_session(&cache);
 
         let location = pos_to_racer_location(params.position);
-        let results = racer::complete_from_file(file_path, location, &session);
+        let results = racer::complete_from_file(&file_path, location, &session);
+        let is_use_stmt = racer::is_use_stmt(&file_path, location, &session);
 
         let code_completion_has_snippet_support =
             ctx.client_capabilities.code_completion_has_snippet_support;
 
-        Ok(
-            results
-                .map(|comp| {
-                    let mut item = completion_item_from_racer_match(&comp);
-                    if code_completion_has_snippet_support {
-                        let snippet = racer::snippet_for_match(&comp, &session);
-                        if !snippet.is_empty() {
-                            item.insert_text = Some(snippet);
-                            item.insert_text_format = Some(InsertTextFormat::Snippet);
-                        }
+        Ok(results
+            .map(|comp| {
+                let mut item = completion_item_from_racer_match(&comp);
+                if is_use_stmt && comp.mtype.is_function() {
+                    item.insert_text = Some(comp.matchstr.to_string());
+                } else if code_completion_has_snippet_support {
+                    let snippet = racer::snippet_for_match(&comp, &session);
+                    if !snippet.is_empty() {
+                        item.insert_text = Some(snippet);
+                        item.insert_text_format = Some(InsertTextFormat::Snippet);
                     }
-                    item
-                })
-                .collect(),
-        )
+                }
+                item
+            }).collect())
     }
 }
 
@@ -351,36 +330,36 @@ impl RequestAction for DocumentHighlight {
         let file_path = parse_file_path!(&params.text_document.uri, "highlight")?;
         let span = ctx.convert_pos_to_span(file_path.clone(), params.position);
 
-        let result = ctx.analysis
+        let result = ctx
+            .analysis
             .find_all_refs(&span, true, false)
             .unwrap_or_else(|_| vec![]);
 
-        Ok(
-            result
-                .iter()
-                .filter_map(|span| {
-                    if span.file == file_path {
-                        Some(lsp_data::DocumentHighlight {
-                            range: ls_util::rls_to_range(span.range),
-                            kind: Some(DocumentHighlightKind::Text),
-                        })
-                    } else {
-                        None
-                    }
-                })
-                .collect()
-        )
+        Ok(result
+            .iter()
+            .filter_map(|span| {
+                if span.file == file_path {
+                    Some(lsp_data::DocumentHighlight {
+                        range: ls_util::rls_to_range(span.range),
+                        kind: Some(DocumentHighlightKind::Text),
+                    })
+                } else {
+                    None
+                }
+            }).collect())
     }
 }
 
+
+
 impl RequestAction for Rename {
-    type Response = WorkspaceEdit;
+    type Response = ResponseWithMessage<WorkspaceEdit>;
 
     fn fallback_response() -> Result<Self::Response, ResponseError> {
-        Ok(WorkspaceEdit {
+        Ok(ResponseWithMessage::Response(WorkspaceEdit {
             changes: None,
             document_changes: None,
-        })
+        }))
     }
 
     fn handle(
@@ -398,26 +377,30 @@ impl RequestAction for Rename {
         let analysis = ctx.analysis;
 
         macro_rules! unwrap_or_fallback {
-            ($e: expr) => {
+            ($e: expr, $msg: expr) => {
                 match $e {
                     Ok(e) => e,
                     Err(_) => {
-                        return Self::fallback_response();
+                        return Ok(ResponseWithMessage::Warn($msg.to_owned()));
                     }
                 }
-            }
+            };
         }
 
-        let id = unwrap_or_fallback!(analysis.crate_local_id(&span));
-        let def = unwrap_or_fallback!(analysis.get_def(id));
+        let id = unwrap_or_fallback!(analysis.crate_local_id(&span), "Rename failed: no information for symbol");
+        let def = unwrap_or_fallback!(analysis.get_def(id), "Rename failed: no definition for symbol");
         if def.name == "self" || def.name == "Self"
             // FIXME(#578)
             || def.kind == data::DefKind::Mod
         {
-            return Self::fallback_response();
+            return Ok(ResponseWithMessage::Warn(format!("Rename failed: cannot rename {}", if def.kind == data::DefKind::Mod { "modules" } else { &def.name })));
         }
 
-        let result = unwrap_or_fallback!(analysis.find_all_refs(&span, true, true));
+        let result = unwrap_or_fallback!(analysis.find_all_refs(&span, true, true), "Rename failed: error finding references");
+
+        if result.is_empty() {
+            return Ok(ResponseWithMessage::Warn("Rename failed: RLS found nothing to rename - possibly due to multiple defs".to_owned()));
+        }
 
         let mut edits: HashMap<Url, Vec<TextEdit>> = HashMap::new();
 
@@ -433,10 +416,13 @@ impl RequestAction for Rename {
         }
 
         if !ctx.quiescent.load(Ordering::SeqCst) {
-            return Self::fallback_response();
+            return Ok(ResponseWithMessage::Warn("Rename failed: RLS busy, please retry".to_owned()));
         }
 
-        Ok(WorkspaceEdit { changes: Some(edits), document_changes: None })
+        Ok(ResponseWithMessage::Response(WorkspaceEdit {
+            changes: Some(edits),
+            document_changes: None,
+        }))
     }
 }
 
@@ -447,13 +433,13 @@ pub enum ExecuteCommandResponse {
 }
 
 impl server::Response for ExecuteCommandResponse {
-    fn send<O: Output>(&self, id: usize, out: &O) {
+    fn send<O: Output>(self, id: server::RequestId, out: &O) {
         // FIXME should handle the client's responses
-        match *self {
+        match self {
             ExecuteCommandResponse::ApplyEdit(ref params) => {
-                let id = out.provide_id() as usize;
+                let id = out.provide_id();
                 let params = ApplyWorkspaceEditParams {
-                        edit: params.edit.clone(),
+                    edit: params.edit.clone(),
                 };
 
                 let request = Request::<ApplyWorkspaceEdit>::new(id, params);
@@ -462,7 +448,7 @@ impl server::Response for ExecuteCommandResponse {
         }
 
         // The formal request response is a simple ACK, though the objective
-        // is the preceeding client requests.
+        // is the preceding client requests.
         Ack.send(id, out);
     }
 }
@@ -479,27 +465,21 @@ impl RequestAction for ExecuteCommand {
         ctx: InitActionContext,
         params: ExecuteCommandParams,
     ) -> Result<Self::Response, ResponseError> {
-        match &*params.command {
-            "rls.applySuggestion" => {
-                apply_suggestion(&params.arguments).map(ExecuteCommandResponse::ApplyEdit)
-            }
-            "rls.deglobImports" => {
-                apply_deglobs(params.arguments, &ctx).map(ExecuteCommandResponse::ApplyEdit)
-            }
-            c => {
-                debug!("Unknown command: {}", c);
-                Err(ResponseError::Message(
-                    ErrorCode::MethodNotFound,
-                    "Unknown command".to_owned(),
-                ))
-            }
+        if params.command.starts_with("rls.applySuggestion") {
+            apply_suggestion(&params.arguments).map(ExecuteCommandResponse::ApplyEdit)
+        } else if params.command.starts_with("rls.deglobImports") {
+            apply_deglobs(params.arguments, &ctx).map(ExecuteCommandResponse::ApplyEdit)
+        } else {
+            debug!("Unknown command: {}", params.command);
+            Err(ResponseError::Message(
+                ErrorCode::MethodNotFound,
+                "Unknown command".to_owned(),
+            ))
         }
     }
 }
 
-fn apply_suggestion(
-    args: &[serde_json::Value],
-) -> Result<ApplyWorkspaceEditParams, ResponseError> {
+fn apply_suggestion(args: &[serde_json::Value]) -> Result<ApplyWorkspaceEditParams, ResponseError> {
     let location = serde_json::from_value(args[0].clone()).expect("Bad argument");
     let new_text = serde_json::from_value(args[1].clone()).expect("Bad argument");
 
@@ -509,9 +489,13 @@ fn apply_suggestion(
     })
 }
 
-fn apply_deglobs(args: Vec<serde_json::Value>, ctx: &InitActionContext) -> Result<ApplyWorkspaceEditParams, ResponseError> {
+fn apply_deglobs(
+    args: Vec<serde_json::Value>,
+    ctx: &InitActionContext,
+) -> Result<ApplyWorkspaceEditParams, ResponseError> {
     ctx.quiescent.store(true, Ordering::SeqCst);
-    let deglob_results: Vec<DeglobResult> = args.into_iter()
+    let deglob_results: Vec<DeglobResult> = args
+        .into_iter()
         .map(|res| serde_json::from_value(res).expect("Bad argument"))
         .collect();
 
@@ -522,17 +506,12 @@ fn apply_deglobs(args: Vec<serde_json::Value>, ctx: &InitActionContext) -> Resul
 
     let text_edits: Vec<_> = deglob_results
         .into_iter()
-        .map(|res| {
-            TextEdit {
-                range: res.location.range,
-                new_text: res.new_text,
-            }
-        })
-        .collect();
+        .map(|res| TextEdit {
+            range: res.location.range,
+            new_text: res.new_text,
+        }).collect();
     // all deglob results will share the same URI
-    let changes: HashMap<_, _> = vec![(uri, text_edits)]
-        .into_iter()
-        .collect();
+    let changes: HashMap<_, _> = vec![(uri, text_edits)].into_iter().collect();
 
     let edit = WorkspaceEdit {
         changes: Some(changes),
@@ -554,11 +533,11 @@ fn make_suggestion_fix_actions(
     code_actions_result: &mut <CodeAction as RequestAction>::Response,
 ) {
     // search for compiler suggestions
-    if let Some(diagnostics) = ctx.previous_build_results.lock().unwrap().get(file_path) {
-        let suggestions = diagnostics
+    if let Some(results) = ctx.previous_build_results.lock().unwrap().get(file_path) {
+        let suggestions = results
             .iter()
-            .filter(|&&(ref d, _)| d.range.overlaps(&params.range))
-            .flat_map(|&(_, ref ss)| ss.iter());
+            .filter(|(diag, _)| diag.range.overlaps(&params.range))
+            .flat_map(|(_, suggestions)| suggestions);
         for s in suggestions {
             let span = Location {
                 uri: params.text_document.uri.clone(),
@@ -568,7 +547,7 @@ fn make_suggestion_fix_actions(
             let new_text = serde_json::to_value(&s.new_text).unwrap();
             let cmd = Command {
                 title: s.label.clone(),
-                command: "rls.applySuggestion".to_owned(),
+                command: format!("rls.applySuggestion-{}", ctx.pid),
                 arguments: Some(vec![span, new_text]),
             };
             code_actions_result.push(cmd);
@@ -585,15 +564,17 @@ fn make_deglob_actions(
     code_actions_result: &mut <CodeAction as RequestAction>::Response,
 ) {
     // search for a glob in the line
-    if let Ok(line) = ctx.vfs
+    if let Ok(line) = ctx
+        .vfs
         .load_line(file_path, ls_util::range_to_rls(params.range).row_start)
     {
         let span = Location::new(params.text_document.uri.clone(), params.range);
 
         // for all indices which are a `*`
         // check if we can deglob them
-        // this handles badly formated text containing multiple "use"s in one line
-        let deglob_results: Vec<_> = line.char_indices()
+        // this handles badly formatted text containing multiple "use"s in one line
+        let deglob_results: Vec<_> = line
+            .char_indices()
             .filter(|&(_, chr)| chr == '*')
             .filter_map(|(index, _)| {
                 // map the indices to `Span`s
@@ -602,11 +583,8 @@ fn make_deglob_actions(
                 span.range.col_end = span::Column::new_zero_indexed(index as u32 + 1);
 
                 // load the deglob type information
-                ctx.analysis.show_type(&span)
-                    .ok()
-                    .map(|ty| (ty, span))
-            })
-            .map(|(mut deglob_str, span)| {
+                ctx.analysis.show_type(&span).ok().map(|ty| (ty, span))
+            }).map(|(mut deglob_str, span)| {
                 // Handle multiple imports from one *
                 if deglob_str.contains(',') || deglob_str.is_empty() {
                     deglob_str = format!("{{{}}}", sort_deglob_str(&deglob_str));
@@ -620,8 +598,7 @@ fn make_deglob_actions(
 
                 // Convert to json
                 serde_json::to_value(&deglob_result).unwrap()
-            })
-            .collect();
+            }).collect();
 
         if !deglob_results.is_empty() {
             // extend result list
@@ -630,7 +607,7 @@ fn make_deglob_actions(
                     "Deglob import{}",
                     if deglob_results.len() > 1 { "s" } else { "" }
                 ),
-                command: "rls.deglobImports".to_owned(),
+                command: format!("rls.deglobImports-{}", ctx.pid),
                 arguments: Some(deglob_results),
             };
             code_actions_result.push(cmd);
@@ -641,7 +618,31 @@ fn make_deglob_actions(
 // Ideally we'd use Rustfmt for this, but reparsing is a bit of a pain.
 fn sort_deglob_str(s: &str) -> String {
     let mut substrings = s.split(',').map(|s| s.trim()).collect::<Vec<_>>();
-    substrings.sort();
+    substrings.sort_by(|a, b| {
+        use std::cmp::Ordering;
+
+        // Algorithm taken from rustfmt (rustfmt/src/imports.rs)
+
+        let is_upper_snake_case = |s: &str| {
+            s.chars()
+                .all(|c| c.is_uppercase() || c == '_' || c.is_numeric())
+        };
+
+        // snake_case < CamelCase < UPPER_SNAKE_CASE
+        if a.starts_with(char::is_uppercase) && b.starts_with(char::is_lowercase) {
+            return Ordering::Greater;
+        }
+        if a.starts_with(char::is_lowercase) && b.starts_with(char::is_uppercase) {
+            return Ordering::Less;
+        }
+        if is_upper_snake_case(a) && !is_upper_snake_case(b) {
+            return Ordering::Greater;
+        }
+        if !is_upper_snake_case(a) && is_upper_snake_case(b) {
+            return Ordering::Less;
+        }
+        a.cmp(b)
+    });
     substrings.join(", ")
 }
 
@@ -729,7 +730,7 @@ fn reformat(
     let path = parse_file_path!(&doc.uri, "reformat")?;
 
     let input = match ctx.vfs.load_file(&path) {
-        Ok(FileContents::Text(s)) => FmtInput::Text(s),
+        Ok(FileContents::Text(s)) => s,
         Ok(_) => {
             debug!("Reformat failed, found binary file");
             return Err(ResponseError::Message(
@@ -746,7 +747,7 @@ fn reformat(
         }
     };
 
-    let range_whole_file = ls_util::range_from_vfs_file(&ctx.vfs, &path);
+    let range_whole_file = ls_util::range_from_file_string(&input);
     let mut config = ctx.fmt_config().get_rustfmt_config().clone();
     if !config.was_set().hard_tabs() {
         config.set().hard_tabs(!opts.insert_spaces);
@@ -754,6 +755,26 @@ fn reformat(
     if !config.was_set().tab_spaces() {
         config.set().tab_spaces(opts.tab_size as usize);
     }
+    if !config.was_set().edition() {
+        match ctx.file_edition(path.clone()) {
+            Some(edition) => {
+                let edition = match edition {
+                    Edition::Edition2015 => RustfmtEdition::Edition2015,
+                    Edition::Edition2018 => RustfmtEdition::Edition2018,
+                };
+                config.set().edition(edition);
+                trace!("Detected edition {:?} for file `{}`", edition, path.display());
+            },
+            None => {
+                warn!("Reformat failed: ambiguous edition for `{}`", path.display());
+
+                return Err(ResponseError::Message(
+                    ErrorCode::InternalError,
+                    "Reformat failed to complete successfully".into(),
+                ));
+            }
+        }
+    }
 
     if let Some(r) = selection {
         let range_of_rls = ls_util::range_to_rls(r).one_indexed();
@@ -762,56 +783,32 @@ fn reformat(
             range_of_rls.row_end.0 as usize,
         );
         let mut ranges = HashMap::new();
-        ranges.insert(FileName::Custom("stdin".to_owned()), vec![range]);
+        ranges.insert(FileName::Stdin, vec![range]);
         let file_lines = FileLines::from_ranges(ranges);
         config.set().file_lines(file_lines);
     };
 
-    let mut buf = Vec::<u8>::new();
-    match format_input(input, &config, Some(&mut buf)) {
-        Ok((summary, ..)) => {
-            // format_input returns Ok even if there are any errors, i.e., parsing errors.
-            if !summary.has_operational_errors() && !summary.has_parsing_errors() {
-                // Note that we don't need to update the VFS, the client
-                // echos back the change to us.
-                let text = String::from_utf8(buf).unwrap();
+    let formatted_text = ctx
+        .formatter()
+        .format(input, config)
+        .map_err(|msg| ResponseError::Message(ErrorCode::InternalError, msg))?;
 
-                if !ctx.quiescent.load(Ordering::SeqCst) {
-                    return Err(ResponseError::Message(
-                        ErrorCode::InternalError,
-                        "Reformat failed to complete successfully".into(),
-                    ))
-                }
+    // Note that we don't need to update the VFS, the client echos back the
+    // change to us when it applies the returned TextEdit.
 
-                // If Rustfmt returns range of text that changed,
-                // we will be able to pass only range of changed text to the client.
-                Ok([
-                    TextEdit {
-                        range: range_whole_file,
-                        new_text: text,
-                    },
-                ])
-            } else {
-                debug!(
-                    "reformat: format_input failed: has errors, summary = {:?}",
-                    summary
-                );
-
-                Err(ResponseError::Message(
-                    ErrorCode::InternalError,
-                    "Reformat failed to complete successfully".into(),
-                ))
-            }
-        }
-        Err(e) => {
-            debug!("Reformat failed: {:?}", e);
-
-            Err(ResponseError::Message(
-                ErrorCode::InternalError,
-                "Reformat failed to complete successfully".into(),
-            ))
-        }
+    if !ctx.quiescent.load(Ordering::SeqCst) {
+        return Err(ResponseError::Message(
+            ErrorCode::InternalError,
+            "Reformat failed to complete successfully".into(),
+        ));
     }
+
+    // If Rustfmt returns range of text that changed,
+    // we will be able to pass only range of changed text to the client.
+    Ok([TextEdit {
+        range: range_whole_file,
+        new_text: formatted_text,
+    }])
 }
 
 impl RequestAction for ResolveCompletion {
@@ -829,23 +826,17 @@ impl RequestAction for ResolveCompletion {
     }
 }
 
-fn racer_coord(
-    line: span::Row<span::OneIndexed>,
-    column: span::Column<span::ZeroIndexed>,
+pub(crate) fn racer_coord(
+    row: span::Row<span::OneIndexed>,
+    col: span::Column<span::ZeroIndexed>,
 ) -> racer::Coordinate {
-    racer::Coordinate {
-        line: line.0 as usize,
-        column: column.0 as usize,
-    }
+    racer::Coordinate { row, col }
 }
 
-fn from_racer_coord(
+pub(crate) fn from_racer_coord(
     coord: racer::Coordinate,
 ) -> (span::Row<span::OneIndexed>, span::Column<span::ZeroIndexed>) {
-    (
-        span::Row::new_one_indexed(coord.line as u32),
-        span::Column::new_zero_indexed(coord.column as u32),
-    )
+    (coord.row, coord.col)
 }
 
 fn pos_to_racer_location(pos: Position) -> racer::Location {
@@ -863,6 +854,39 @@ fn location_from_racer_match(a_match: &racer::Match) -> Option<Location> {
     })
 }
 
+impl RequestAction for CodeLensRequest {
+    type Response = Vec<CodeLens>;
+
+    fn fallback_response() -> Result<Self::Response, ResponseError> {
+        Err(ResponseError::Empty)
+    }
+
+    fn handle(
+        ctx: InitActionContext,
+        params: Self::Params,
+    ) -> Result<Self::Response, ResponseError> {
+        let mut ret = Vec::new();
+        if ctx.client_supports_cmd_run {
+            let file_path = parse_file_path!(&params.text_document.uri, "code_lens")?;
+            for action in collect_run_actions(&ctx, &file_path) {
+                let command = Command {
+                    title: action.label,
+                    command: "rls.run".to_string(),
+                    arguments: Some(vec![serde_json::to_value(&action.cmd).unwrap()]),
+                };
+                let range = ls_util::rls_to_range(action.target_element);
+                let lens = CodeLens {
+                    range,
+                    command: Some(command),
+                    data: None,
+                };
+                ret.push(lens);
+            }
+        }
+        Ok(ret)
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
@@ -874,5 +898,9 @@ mod test {
         assert_eq!(sort_deglob_str("a, b"), "a, b");
         assert_eq!(sort_deglob_str("b, a"), "a, b");
         assert_eq!(sort_deglob_str("foo, bar, baz"), "bar, baz, foo");
+        assert_eq!(
+            sort_deglob_str("Curve, curve, ARC, bow, Bow, arc, Arc"),
+            "arc, bow, curve, Arc, Bow, Curve, ARC",
+        );
     }
 }
